#!/usr/bin/env python
'''
-------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
-------------------------------------------------------------------------------
	Set of classes for handling application configuration persistence.
	So that persistence storage and format can change without affecting
	therest o the code application.
	At this first release, it is based on standard xml python package.
	Further implementations are in the todo list: 
	(MySql, DB2, Oracle, Directory server, gconf, ect.)

'''

__version__ = "0.1.1"
__author__  = "Oreste Notelli <oreste.notelli@sischema.com>"

import os

def home_configuration_dirname(application_name):
    '''returns a dir name composed as follows
       <home directory>/<application_name>'''
    return os.path.join (os.environ['HOME'], application_name)

def home_configuration_filepath(application_name, config_filename):
    '''returns a path composed as follows:
       <home directory>/<application_name>/<config_filename>'''
    return os.path.join (   os.environ['HOME'], 
                            home_configuration_dirname(application_name), 
                            config_filename)

def _configuration_file(application_name, configuration_filename):
    '''returns a path composed as follows:
       <home directory>/<application_name>/<config_filename>
       and creates all not existent directories.'''
    confdir = home_configuration_dirname(application_name);
    if not os.path.exists(confdir): os.makedirs(confdir)
    return home_configuration_filepath(application_name,configuration_filename)

import xml.marshal.generic

def _xml_load (filename, defaults = None):
    '''reads an object previously saved in the given file
       defaults are used in case of missing file.'''
    print "_xml_load "+filename
    try:
        return xml.marshal.generic.load(open(filename, "r"))
    except IOError:
        return defaults

def _xml_save(filename, value):
    xml.marshal.generic.dump(value, open(filename, "w"))
    print "_xml_save "+filename

class homedirXMLfileconfiguration:
    def __init__(self, main_key):
        self.__main_key=main_key

    def load(self, configurationkey, defaults = None):
        fn = _configuration_file(self.__main_key, configurationkey)
        return _xml_load(fn,defaults)
    def save(self, configurationkey, val):
        fn = _configuration_file(self.__main_key, configurationkey)
        return _xml_save(fn,val)


###############################################################################
# GUI
import gtkforms

def __emptycallback():
    pass

def __gtk_create_dialog_input(fields,
                              response_callback =__emptycallback,
                              icon = None,
                              flags = 0,
                              Parent = None):
    dialog = gtkforms\
              .create_gtk_dialog_from_dict(fields,
                                          parent=Parent,
                                          flags=flags,
                                          response_callback=response_callback)
    if (icon != None):
        dialog.set_icon(icon)
    return dialog

def gtk_modal_dialog_input( fields,
                            icon = None,
                           ):
        d = __gtk_create_dialog_input(fields,
                                      icon=icon, 
                                      flags=gtk.DIALOG_MODAL)
        return (d.run() == gtk.RESPONSE_OK)

def gtk_dialog_input(fields,
                     callback = __emptycallback,
                     icon = None):
    '''shows a dialog window composed by GTK widgets mapped to given 
	   variables.		
       fields is a dictionary where the key is used for the label.
       each field value to a GTK widget.
	   variable type is preserved.'''
    __gtk_create_dialog_input(fields,
                              callback,
                              icon
                              )\
                                      .show_all()
     
###############################################################################
# Examples
import gtk
import sys

def onclose():  
    print (__defaults)
    sys.exit()

__defaults = {'width':100,
              'font color':'blue',
            'font size': 8,
            'true or false': False}

if (__name__ == "__main__"):
    w = gtk_dialog_input(__defaults, onclose)
    gtk.main()
    sys.exit()
