'''
--------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
--------------------------------------------------------------------------------
   Extractors.
   Set of functionoids (functions or callable objects) used to extract
   value from the given text.
'''

__version__ = "0.1.1"
__author__  = "Oreste Notelli <oreste.notelli@sischema.com>"

################################################################################

def simple_extractor (text, bpos, epos):
    ''' 
        returns the text comprise bpos and epos (excluded)
    '''
    return text[bpos:epos]

################################################################################

import xml.sax.saxutils
class __html_cleaner :
    def handle_char(self, char):
        if(char == '<'):
            self.in_tag = True
        elif (char == '>'):
            self.in_tag = False
        elif not self.in_tag : self.text = self.text+char

    def feed (self, text):
        self.text = ""
        self.in_tag = False
        for char in text:
            self.handle_char(char)

def html_cleaner(text, bpos, epos):
    ''' 
        returns the text comprise bpos and epos (excluded), purged from HTML
        elements.
    '''
    parser = __html_cleaner()
    parser.feed(simple_extractor(text, bpos, epos))
    return xml.sax.saxutils.unescape(parser.text, {'&#39;':'\'', '&quot;':'"', "&apos;":"'"})

################################################################################
import string

def ascii (text, bpos, epos):
    new_text = simple_extractor(text, bpos, epos)
    txt = ''
    for c in new_text:
        if string.printable.find(c) > 0:
            txt = txt + c
    return txt

################################################################################

def trimmer (text, bpos, epos):
    ''' 
        returns the text comprise bpos and epos (excluded) without rigth and
        left blanks
    '''
    return string.strip(simple_extractor(text, bpos, epos))


################################################################################

class composite:
    ''' 
        executes the given extractors the text comprise bpos and epos (excluded)
    '''
    def __init__(self, extractors):
        self.__extractors = extractors
    def __call__(self, text, bpos, epos):
        result = text
        p1 = bpos
        p2 = epos
        for extractor in self.__extractors:
            result = extractor(result, p1, p2)
            p1 = 0
            p2 = len(result)
        return result

################################################################################

class string_composition:
    ''' 
        extractor callable class. It add the given strings to the extracted
        text
    '''
    def __init__(self, begin, end):
        self.begin = begin
        self.end = end
    def __call__(self, text, bpos, epos):
        return  self.begin + simple_extractor(text, bpos, epos)+ self.end

