#!/usr/bin/env python

'''
-------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
-------------------------------------------------------------------------------
'''

description = 'Editable cells'

__version__ = "0.1.1"
__author__  = "Oreste Notelli <oreste.notelli@sischema.com>"

import gobject
import gtk
from gtk import TRUE, FALSE
import utils
import feeds
import configuration

COLUMN_ENABLED = 0
COLUMN_NAME = 1
COLUMN_EDITABLE = 2
COLUMN_TYPE = 3

class  gtk_selection_dialog:
    def __init__(self,parent, selection_list):
        window = gtk.Dialog( flags = gtk.DIALOG_MODAL,
                             buttons = ((gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_OK,     gtk.RESPONSE_OK, )))

        s = utils.selector (selection_list)
        window.vbox.add(s.get_widget())
        window.set_icon(parent.get_icon())

        self.response = s.selected
        window.vbox.show_all()
        window.connect("response", self.__on_resonse)
        if (window.run() ==  gtk.RESPONSE_OK):
            self.response = s.selected
        else:
            self.response = None
    def __on_resonse(self, window, resp):
        window.hide()
        

def create_model(configuration, feed_list):

    # create model
    model = gtk.ListStore(gobject.TYPE_BOOLEAN,
                          gobject.TYPE_STRING,
                          gobject.TYPE_BOOLEAN)

    # insert feeds
    l = []
    l = configuration.load("feeds", l)
    if len (l) == 0:
        for feed in feed_list:
            iter = model.append()
            model.set(iter,
                    COLUMN_ENABLED, TRUE,
                    COLUMN_NAME, feed.name(),
                    COLUMN_EDITABLE, TRUE)
            l.append ([TRUE, feed.name(), TRUE, feed])
    else:
        for item in l:
            iter = model.append()
            model.set(iter,
                    COLUMN_ENABLED, item[COLUMN_ENABLED],
                    COLUMN_NAME, item[COLUMN_NAME],
                    COLUMN_EDITABLE, item[COLUMN_EDITABLE])
    return (model, l)
   
class feeds_config_dialog:
    def __init__(self, configuration, icon, callback = utils.empty_function):
        self.__callback = callback
        window = gtk.Window()
        window.set_title('Feed configuration')
        window.set_icon(icon)
        window.set_border_width(5)
        vbox = gtk.HBox()
        window.add(vbox)
        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC,
                      gtk.POLICY_AUTOMATIC)
        vbox.pack_start(sw)

        # create model
        model, self.__feeds = create_model(configuration,
                                           feeds.get_default_feeds())
        self.model = model
        self.window = window

        # create tree view
        treeview = gtk.TreeView(model)
        treeview.set_rules_hint(TRUE)
        treeview.set_headers_visible(TRUE)
        self.treeview = treeview
        selection = treeview.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)

        self.add_columns()

        sw.add(treeview)

        # some buttons
        hbox = gtk.VBox()
        vbox.pack_start(hbox, FALSE, FALSE)

        button = gtk.Button('Add item', gtk.STOCK_ADD)
        #button.connect("button_press_event", self.add_item, self.model)
        button.connect('clicked', self.__add_item, self.model)
        hbox.pack_start(button)
    
        button = gtk.Button('Remove item', gtk.STOCK_REMOVE)
        button.connect('clicked', self.__remove_item, treeview)
        hbox.pack_start(button)

        button = gtk.Button('Edit item', gtk.STOCK_PROPERTIES)
        button.connect('clicked', self.__edit_item, treeview)
        hbox.pack_start(button)

        button = gtk.Button('Move up', gtk.STOCK_GO_UP)
        button.connect('clicked', self.__moveup_item, treeview)
        hbox.pack_start(button)

        button = gtk.Button('Move down', gtk.STOCK_GO_DOWN)
        button.connect('clicked', self.__movedown_item, treeview)
        hbox.pack_start(button)

        button = gtk.Button('Move down', gtk.STOCK_CLOSE)
        button.connect('clicked', self.__on_close, window, configuration)
        hbox.pack_start(button)

        window.connect('destroy',
                       lambda win: self.__on_close(None, win, configuration) )

        window.set_default_size(330, 220)
        
        window.show_all()

    def add_columns(self):
        model = self.treeview.get_model()

        # number column
        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self.__cell_toggled, model)
        renderer.set_data('column', COLUMN_ENABLED)
        self.treeview.insert_column_with_attributes(-1, 'Enabled', renderer,
                                           active=COLUMN_ENABLED)
        # product column
        renderer = gtk.CellRendererText()
        renderer.set_data('column', COLUMN_NAME)

        self.treeview.insert_column_with_attributes(-1, 'Name', renderer,
                                           text=COLUMN_NAME)

    def __add_item(self, button, model):
        types = feeds.types()
        dtypes = dict()
        for parser_type in types:
            dtypes [parser_type.name] = parser_type
        if (len(types) >1):
            d = gtk_selection_dialog(self.window, dtypes.keys())    
            if (d.response == None): return
            selected_type = dtypes[d.response]
        else:
            selected_type = types[0]
        new_feed = selected_type.create_instance()
        result = self.__feed_edit(new_feed)
        if  result:
            item = [TRUE, new_feed.properties["name"], TRUE, new_feed]
            iter = model.append()
            model.set(iter,
                      COLUMN_ENABLED, item[0],
                      COLUMN_NAME, item[1],
                      COLUMN_EDITABLE, item[2])
            self.__feeds.append(item)
    
    def __edit_item(self, button, treeview):
        selection = treeview.get_selection()
        selected = selection.get_selected()
        if selected:
            model, iter = selected
            i = model.get_path(iter)
            index,  = i
            selected_type = self.__feeds[index][COLUMN_TYPE]
            result = self.__feed_edit(selected_type)
            self.__feeds[index][COLUMN_NAME] = selected_type.properties["name"]
            model.set(iter, COLUMN_NAME, self.__feeds[index][COLUMN_NAME])

    def __feed_edit(self, feed):
        return configuration.\
                    gtk_modal_dialog_input(
                           feed.properties)
                           
    def __remove_item(self, button, treeview):
        selection = treeview.get_selection()
        selected = selection.get_selected()
        if selected:
            model, iter = selected
            i = model.get_path(iter)[0]
            model.remove(iter)
            self.__feeds.pop(i)

    def __on_close (self, button, window, configuration):
        #TODO: save feeds
        import types
        # patch for booleans in xml marshaler
        for elem in self.__feeds:
            if (elem[0] == True): 
                elem[0] = 1
            else:
                elem[0] = 0
            if (elem[2] == True): 
                elem[2] = 1
            else:
                elem[2] = 0
        configuration.save("feeds", self.__feeds)
        window.hide()
        self.__callback()

    def __moveup_item (self, button, treeview):
        selection = treeview.get_selection()
        selected = selection.get_selected()
        if selected:
            model, iter = selected
            i = model.get_path(iter)[0]
            if (i > 0):
                temp = self.__feeds[i][:]
                self.__feeds[i] = self.__feeds[i-1]
                self.__feeds[i-1]= temp
                prev = model.get_iter(i-1, )
                model.swap(iter, prev)

    def __movedown_item (self, button, treeview):
        selection = treeview.get_selection()
        selected = selection.get_selected()
        if selected:
            model, iter = selected
            i = model.get_path(iter)[0]
            if (i < (len (self.__feeds) -1)):
                temp = self.__feeds[i][:]
                self.__feeds[i] = self.__feeds[i+1]
                self.__feeds[i+1]= temp
                next = model.iter_next(iter)
                model.swap(iter, next)

    def __cell_toggled (self, cell, arg, model):
        row = self.__feeds[int(arg)]
        row[COLUMN_ENABLED] = not row[COLUMN_ENABLED]
        iter = model.get_iter_from_string(arg)
        column = cell.get_data("column")
        if column == COLUMN_ENABLED:
            model.set(iter, column, row[column])

def main():
    # create window, etc
    feeds_config_dialog(configuration\
                        .homedirXMLfileconfiguration("GNUsTicker"),
                        None)
    gtk.main()
    
if __name__=="__main__":
    main()
