#!/usr/bin/env python
'''
--------------------------------------------------------------------------------
    Copyright  (C) 2005, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
--------------------------------------------------------------------------------
'''
__version__ = "0.0.1"
__author__  = "Oreste Notelli <oreste.notelli@sischema.com>"

import gtk
import GNUsTicker
import os
import thread

class news_widget:
    def __init__(self):
        self.__configuration = {'scroll delay':100, 
                              'scrolling step':1,
                              'skip step':50,
                              'stop on mouse over':False}#, 'separator':'  -  '}
        self.__called = False
        self.__lock = thread.allocate_lock()
        self.__product_assigned = False;
        self.__control_created = False;

    def activate(self, applet, container):
        self.__applet = applet
        self.__container = container
        self.__box = gtk.HBox()
        self.__tooltips = gtk.Tooltips()

        arrow_left = gtk.Arrow(gtk.ARROW_LEFT, gtk.SHADOW_NONE);
        self.__button_left = gtk.Button()
        self.__button_left.add(arrow_left)
        self.__button_left.connect("clicked", self.__on_left)
        self.__tooltips.set_tip(self.__button_left, "Skip forward")

        arrow_right = gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_NONE);
        self.__button_right = gtk.Button()
        self.__button_right.add(arrow_right)
        self.__button_right.connect("clicked", self.__on_right)
        self.__tooltips.set_tip(self.__button_right, "Skip backward")

        self.__box.pack_start(self.__button_left, False, False)
        self.__view = GNUsTicker.newswidget(self.__box)
        self.__view.set_scroll_delay(self.__configuration['scroll speed'])
        self.__view.set_scrolling_step(self.__configuration['scrolling step'])
        self.__view.set_stop_on_mouse_over(self.__configuration['stop on mouse over'])

        self.__box.pack_start(self.__button_right,False, False)
        container.add(self.__box)
        #self.__view.set_click_callback(self.__onclick)
        # synchronizated code
        self.__lock.acquire()
        print("__control_creating")
        if (self.__product_assigned):
            print("__product_assigned")
            for title, product in self.__products.iteritems():
                self.__set_feed_product(product)
        self.__control_created = True
        self.__lock.release()
        print("__control_created")
        
    def set_feed_product(self, product):
        # synchronizated code
        #self.__lock.acquire()
        self.__set_feed_product(product)
        #self.__lock.release()

    def __set_feed_product(self, product):
        if (self.__control_created):
            n = GNUsTicker.newspage(str(product.title), "no text")
            for news in product.elements:
                n.add(news)
            self.__view.set_feed(n);
        else:
            print("not __control_created")
            self.__products[product.title]= product
        self.__product_assigned = True
        
    def set_configuration(self, newconf):
        self.__configuration = newconf
        if (self.__configuration['stop on mouse over'] == 1):
            self.__configuration['stop on mouse over'] = True
        else:
            self.__configuration['stop on mouse over'] = False

    def get_configuration(self):
        return self.__configuration

    def get_configuration_to_save(self):
        configuration_to_save = self.__configuration.copy()
        if (configuration_to_save['stop on mouse over']):
            configuration_to_save['stop on mouse over'] = 1
        else:
            configuration_to_save['stop on mouse over'] = 0
        return configuration_to_save

    def remove_all_products(self):
        self.__view.remove_all_feeds()

    def on_configuration_changed(self):
        print (self.__configuration)
        self.__view.set_scroll_delay(self.__configuration['scroll speed'])
        self.__view.set_scrolling_step(self.__configuration['scrolling step'])
        self.__view.set_stop_on_mouse_over(self.__configuration['stop on mouse over'])

    def __on_left(self, button):
        self.__view.scroll(- (self.__configuration['skip step']))

    def __on_right(self, button):
        self.__view.scroll((self.__configuration['skip step']))

    def remove (self):
        self.__container.remove(self.__box)
    
