/*
-------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
-------------------------------------------------------------------------------
*/
#ifndef _newpage_
#define _newpage_

#include <vector>
#include <string>
#include <algorithm>
#include <iterator>
#include <boost/shared_ptr.hpp>

namespace GtkExtensions 
{
typedef std::string string;
class newspage
{
public:
    class element
    {
    public:
        element(string title, string description );
        element (const element& elem);
        element ();
        element& operator= (const element& elem);
        virtual void action() {printf("newspage::element::action()\n");};
        string title, description;
    };
    typedef std::vector<boost::shared_ptr<element> > elements;
    typedef elements::iterator iterator;
    typedef elements::const_iterator const_iterator;

    newspage(string title, string description);
    newspage();
    newspage(const newspage&);
    newspage& add_element(boost::shared_ptr<element> elem);
    void python_add_element( boost::shared_ptr<element> elem);
    string  title, description;
    iterator begin();
    iterator end();
private:
    elements _elements;
};

//INLINES
inline newspage::element::element(string arg_title, string arg_description)
:title(arg_title), 
description(arg_description)
{
}

inline newspage::element::element (const element& elem):
title(elem.title), 
description(elem.description)
{
}

inline newspage::element::element ()
{}

inline newspage::element& 
newspage::element::operator= (const newspage::element& elem)
{
    title = elem.title;
    description = elem.description;
    return *this;
}

inline newspage::newspage(
                    string arg_title, 
                    string arg_description
                    ):title(arg_title), 
                     description(arg_description)
                    
{}


inline void newspage::python_add_element( boost::shared_ptr<element> elem)
{
    _elements.push_back(elem);
}

inline newspage& newspage::add_element( boost::shared_ptr<element> elem)
{
    _elements.push_back(elem);
    return *this;
}

inline newspage::iterator newspage::begin()
{
    return _elements.begin();
}

inline newspage::newspage()
{
}

inline newspage::newspage(const newspage& arg):
    title(arg.title),
    description(arg.description)
{
    std::insert_iterator<elements> in(_elements, _elements.begin());
    std::copy (arg._elements.begin(), arg._elements.end(), in);
}

inline newspage::iterator newspage::end()
{
    return _elements.end();
}

}
#endif // _GtkExtensions_ScrollingTextEx_
