/*
-------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
-------------------------------------------------------------------------------
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtkmm/enums.h>
#include <gtkmm/window.h>
#include <gtkmm/arrow.h>
#include <gtkmm.h>
#include <gtk/gtkdrawingarea.h>
#include <iostream>
#include <string>
#include <gtk/gtk.h>
#include <boost/python/class.hpp>
#include <boost/python/iterator.hpp>
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <boost/python/dict.hpp>
#include <boost/python/wrapper.hpp>
#include <boost/python/override.hpp>
#include <boost/python/errors.hpp>
#include <boost/ref.hpp>
#include <sstream>
#include <map>
#include <utility>
#include <iterator>

#include "scrollingframe.hh"

extern "C"
{
GtkContainer* getContainer(PyObject* object);
}
#include "newspage.hh"
using namespace GtkExtensions;

namespace { 
    class newswidget 
    {
    

    public:
    struct element :
        public newspage::element, 
        public boost::python::wrapper<newspage::element>
    {
        element(
                    std::string title, 
                    std::string description
                    ):
        newspage::element(title, description)
        {
        }
        virtual void action()
        {
            try
            {
                if (boost::python::override action = this->get_override("action"))
                    action();
                else
                    newspage::element::action();
            }
            catch(boost::python::error_already_set)
            {
                printf("ERROR on action\n");
            }
        }

        void default_action() { this->newspage::element::action(); }
    };
        newswidget()
        {
        }
        newswidget(const newswidget& copy)
        {
            // TODO: dangerous: never should happen
            printf("DANGER: newswidget(const newswidget& copy) never should happen\n");
            frame = copy.frame;
        }
        newswidget(boost::python::api::object parent):
            frame(new GtkExtensions::ScrollingFrame())        
        {
            add_parent(parent);
        }
        
        void set_scroll_delay(int delay)
        {
            frame->set_scroll_delay(delay);
        }

        void set_scrolling_step(int delay)
        {
            frame->set_scrolling_step(delay);
        }

        void remove_all_feeds()
        {
            frame->remove_all_feeds();
        }

/*        inline void newspage::python_set_feed(boost::python::api::object elem)
        {
            boost::python::dict d = 
                            extract<boost::python::dict>(elem.attr('__dict__'));
            
        }
*/
        void set_feed(newspage& news_page)
        {
            frame->set_news_page(news_page);
        }

        void set_pause (bool pause)
        {
            frame->set_pause(pause);
        }

        void set_stop_on_mouse_over(bool flag)
        {
            frame->set_stop_on_mouse_over(flag);
        }

        void scroll(int value)
        {
            frame->scroll(value);
        }

        ~newswidget()
        {
            frame;
        }

    private:
        void add_parent(boost::python::api::object parent)
        {
            applet_fill(getContainer(parent.ptr()));
        }


        void applet_fill (GtkContainer* container)
        {
            gtk_container_add(container, GTK_WIDGET(frame->gobj()) );
        }
        Glib::RefPtr<GtkExtensions::ScrollingFrame> frame;
		boost::python::api::object _callback;
  };    

}

// TODO:: It seems to be necessary to create a Gtk::Main object
// we temporary place it here as a static
Gtk::Main kit(NULL, NULL);

BOOST_PYTHON_MODULE(GNUsTicker)
{
    using namespace boost::python;
    class_<newswidget::element,boost::noncopyable>("news", 
                                             init<std::string,
                                                  std::string>())
        .def_readwrite("title", &newswidget::element::title)
        .def_readwrite( "description", 
                        &newswidget::element::description)
        .def("action", &GtkExtensions::newspage::element::action, &newswidget::element::default_action);

    class_<GtkExtensions::newspage,boost::noncopyable>("newspage", 
                                    init<std::string,
                                         std::string>())
        .def_readwrite("title", &GtkExtensions::newspage::title)
        .def_readwrite("description", &GtkExtensions::newspage::description)
        .def("add", &GtkExtensions::newspage::python_add_element)
        .def("news", range( &GtkExtensions::newspage::begin, 
                            &GtkExtensions::newspage::end));    

    class_<newswidget,boost::noncopyable>("newswidget", init<object>())
        .def("set_feed", &newswidget::set_feed)
        .def("remove_all_feeds", &newswidget::remove_all_feeds)
        .def("scroll", &newswidget::scroll)
        .def("set_scroll_delay", &newswidget::set_scroll_delay)
        .def("set_scrolling_step", &newswidget::set_scrolling_step)
        .def("set_pause", &newswidget::set_pause)
        .def("set_stop_on_mouse_over", &newswidget::set_stop_on_mouse_over);
    if (!Glib::thread_supported())
        Glib::thread_init ();

}
