/*
--------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
--------------------------------------------------------------------------------
*/
#ifndef _GtkExtensions_ScrollingFrame_
#define _GtkExtensions_ScrollingFrame_

#include "scrollingtextex.hh"
#include "newspage.hh"
#include <map>

namespace GtkExtensions 
{

class ScrollingFrame :  public Gtk::Frame,
                        public ScrollingTextEx::callback
{
private:
    class part :public ScrollingTextEx::part
    {
    public:
        //ctors 'n' dtor
        part(   Gtk::EventBox& event, 
                Gtk::Tooltips& tooltips, 
                 boost::shared_ptr<newspage::element> element 
		     );
        virtual ~part(){};
        // methods
        virtual Glib::ustring    get_text() const;
        virtual void             over(); 
        virtual void             action();
    private:
        // attributes
        boost::shared_ptr<newspage::element>    _elem;
        Gtk::EventBox&                          _event;
        Gtk::Tooltips&                          _tooltips;
    };
public:
    //ctors 'n' dtor
    ScrollingFrame();
    ScrollingFrame&    set_news_page(newspage& news_arg);
    //methods
    void            scroll                  (int value);
    void            set_scroll_delay        (int delay);
    void            set_scrolling_step      (int step);
    void            set_pause               (bool pause);
    void            set_stop_on_mouse_over  (bool flag);
    void            remove_all_feeds        ();

private:
    //override
    virtual void restart();
    //attributes
    Gtk::EventBox                   _event;
    Gtk::Tooltips                   _tooltips;
    std::map<std::string,newspage>  _news;
    ScrollingTextEx                 _scroll;
//    callback_ptr	               _callback_handler;
};

//INLINES

inline ScrollingFrame::part::part (Gtk::EventBox& event, 
                            Gtk::Tooltips& tooltips, 
                            boost::shared_ptr<newspage::element> elem):
_elem(elem),
_event(event),
_tooltips(tooltips)
{
}

inline Glib::ustring ScrollingFrame::part::get_text() const
{
    return _elem->title;
}

inline void ScrollingFrame::part::over()
{
    _tooltips.set_tip(_event, _elem->description);
}

inline void ScrollingFrame::part::action()
{
    //printf("ScrollingFrame::part::action begin\n");
    _elem->action();
    //_callback_handler->onclick(_elem->url);
    //printf("ScrollingFrame::part::action end\n");
}

inline void ScrollingFrame::scroll(int value)
{
    _scroll.scroll(value);
}

inline void ScrollingFrame::set_scroll_delay(int delay)
{
    _scroll.set_scroll_delay(delay);
}

inline void ScrollingFrame::set_scrolling_step  (int step)
{
    _scroll.set_scrolling_step(step);
}

inline void ScrollingFrame::set_pause(bool pause)
{
    _scroll.set_pause(pause);
}

inline void ScrollingFrame::set_stop_on_mouse_over(bool flag)
{
    _scroll.set_stop_on_mouse_over(flag);
}

inline ScrollingFrame::ScrollingFrame():
    _scroll(this)
{
    //printf("ScrollingFrame::ScrollingFrame() begin\n");
    set_shadow_type(Gtk::SHADOW_IN);
    add(_event);
    _event.add(_scroll);
    //printf("ScrollingFrame::ScrollingFrame() end\n");
}

}
#endif // _GtkExtensions_ScrollingFrame_
