/*
--------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
--------------------------------------------------------------------------------
*/
#include "scrollingtext.hh"
namespace GtkExtensions 
{
//ScrollingText
ScrollingText::ScrollingText(const ScrollingText::properties& props):
    _first_text(true), 
    _props (props),
    _pos(0),
    _text_width(0),
    _paused(false),
    _stop_on_mouse_over(false),
    _previous_flag(false)
{
	add_events(Gdk::LEAVE_NOTIFY_MASK |Gdk::ENTER_NOTIFY_MASK);
    
    signal_configure_event().connect(sigc::mem_fun(this, 
                                                &ScrollingText::on_configure)); 
    signal_expose_event().connect(sigc::mem_fun(this, 
                                                &ScrollingText::on_expose)); 
}

void  ScrollingText::set_scroll_delay (int delay)
{
    if (_connection)
        _connection.disconnect();
    _connection = Glib::signal_timeout()
        .connect(sigc::mem_fun(this, &ScrollingText::repaint),
                 delay);
    _props.set_sleep_time(delay);
}

void ScrollingText::set_text(Glib::ustring text)
{
    if (_first_text)
    {   
        set_scroll_delay (get_sleep_time());
    }
    Glib::RecMutex::Lock lock(mutex);
    Glib::RefPtr<Pango::Layout> layout = get_layout();
    layout->set_text(text.c_str());
    _text_width = layout->get_pixel_logical_extents().get_width();
    //init_cur_pos();
    set_cur_pos(get_allocation().get_width());
    _first_text = false;
}

bool ScrollingText::on_configure (GdkEventConfigure* event)
{
    _pixmap = Gdk::Pixmap::create(get_window(),
		          get_allocation().get_width(),
		          get_allocation().get_height(),
		-1);
  	_pixmap->draw_rectangle(get_style()->get_bg_gc(get_state()),
      true,
      0, 0,
      get_allocation().get_width(),
      get_allocation().get_height());

    return true;
}

bool ScrollingText::on_expose (GdkEventExpose* event)
{
    gdk_draw_pixmap(
        GDK_DRAWABLE(get_window()->gobj()),
		get_style()->get_fg_gc(get_state())->gobj(),
		_pixmap->gobj(),
		event->area.x, event->area.y,
		event->area.x, event->area.y,
		event->area.width,event->area.height);
        return false;
}

void ScrollingText::on_unrealize()
{
    if (_connection)
        _connection.disconnect();
}

bool ScrollingText::repaint()
{
    Glib::RecMutex::Lock lock(mutex);
    if (_paused) return true;
    Glib::RefPtr<Gdk::GC> gc = get_gc(); 
    Gdk::Rectangle allocation = get_allocation();
    int width = allocation.get_width();
    gc->set_rgb_fg_color(get_bg_color());
    Glib::RefPtr<Gdk::Pixmap> pixmap =get_pixmap();
    pixmap->draw_rectangle(gc, true, 0,0,width,
            get_layout()->get_pixel_logical_extents().get_width());
            //allocation.get_height());
    if (!is_text_visible()) init_cur_pos();
    gc->set_rgb_fg_color(get_fg_color());
    pixmap->draw_layout (gc, get_cur_pos() ,0, get_layout() );
    decrease_pos();
    queue_draw();
    return true;
}

Glib::RefPtr<Gdk::GC> ScrollingText::get_gc()
{
    if (!_GC)
    {
        _GC = Gdk::GC::create(get_window());
    }
    return _GC;
}
Glib::RefPtr<Pango::Layout> ScrollingText::get_layout()
{
    if (!_layout)
        _layout = create_pango_layout("");
    return _layout;
}

bool ScrollingText::on_leave_notify_event(GdkEventCrossing* event)
{
    Glib::RecMutex::Lock lock(mutex);
    if (_stop_on_mouse_over)
    {
        _paused = _previous_flag;
    }
}

bool ScrollingText::on_enter_notify_event(GdkEventCrossing* event)
{
    Glib::RecMutex::Lock lock(mutex);
    if (_stop_on_mouse_over)
    {
        _previous_flag = _paused;
        _paused = true;
    }
}

}
