/*
--------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
--------------------------------------------------------------------------------
*/
#ifndef _GtkExtensions_ScrollingText_
#define _GtkExtensions_ScrollingText_
#define G_THREADS_ENABLED
#include <gtkmm.h>
#define G_THREADS_ENABLED


namespace GtkExtensions 
{
    template <class  T>
    T* no_const(const T * t )
    {
        return const_cast<T*> (t);
    }
    
/*
 *ScrollingText
 * example:
 *  ScrollingText scrollinfText(  ScrollingText::properties()
                    .set_sleep_time(100));
    scrollinfText.set_text("hello world");
 */
class ScrollingText : public Gtk::DrawingArea
{
public:
    class properties
    {
    public:
        properties(const ScrollingText::properties& arg);
        properties();
        properties&         set_sleep_time      (int sleep_time);
        properties&         set_scrolling_step  (int step);
        //properties&         set_bg_color        (Gdk::Color& color);
        //properties&         set_fg_color        (Gdk::Color& color);
        //const Gdk::Color&   get_bg_color        () const;
        //const Gdk::Color&   get_fg_color        () const;
        int                 get_sleep_time      () const;
        int                 get_scrolling_step  () const;
    private:
        Gdk::Color _bgColor, _fgColor;
        int _sleep_time;
        int _scrolling_step;
    };

    ScrollingText(const ScrollingText::properties& props 
                  = ScrollingText::properties());
    void                        set_text                (Glib::ustring text);
    void                        scroll                  (int value);
    Glib::ustring               get_text();
    void                        set_scroll_delay        (int delay);
    void                        set_scrolling_step      (int step);
    void                        set_pause               (bool pause);
    void                        set_stop_on_mouse_over  (bool flag);
    virtual void                reset();

protected:
    bool                        on_configure    (GdkEventConfigure* event);
    bool                        on_expose       (GdkEventExpose* event);
    virtual void                set_cur_pos     (int value);
    bool                        repaint();
    int                         get_text_width() const;
    int                         get_cur_pos() const;
    int                         get_width() const;
    void                        decrease_pos();
    virtual void                init_cur_pos();
    // Overrides
    virtual void                on_unrealize();
    virtual bool                on_leave_notify_event(GdkEventCrossing* event);
    virtual bool                on_enter_notify_event(GdkEventCrossing* event);
private:
    const Gdk::Color&           get_bg_color() const;
    const Gdk::Color&           get_fg_color() const;
    int                         get_sleep_time() const;
    int                         get_scrolling_step () const;
    bool                        is_text_visible() const; 
    Glib::RefPtr<Gdk::GC>       get_gc();
    Glib::RefPtr<Pango::Layout> get_layout();
    Glib::RefPtr<Gdk::Pixmap>   get_pixmap();
    //data members
    bool _paused;
    Glib::RefPtr<Gdk::Pixmap> _pixmap;
    Glib::RefPtr<Gdk::GC> _GC;
    Glib::RefPtr<Pango::Layout> _layout;
    Gdk::Color _bgColor, _fgColor;
    bool _first_text, _stop_on_mouse_over, _previous_flag;
    sigc::connection _connection;
    properties _props;
    int _pos, _text_width;
    Glib::RecMutex mutex;
};

//INLINES
inline ScrollingText::properties::properties()
{
    _sleep_time = 100;
    _scrolling_step = 1;
    //_bgColor.set_rgb(60000,60000,60000);
    //_fgColor.set_rgb(0,0,0);
}

inline ScrollingText::properties::properties(
    const ScrollingText::properties::properties& arg):
    //_bgColor(arg._bgColor),
    //_fgColor(arg._fgColor),
    _sleep_time(arg._sleep_time),
    _scrolling_step(arg._scrolling_step)
{
}

inline ScrollingText::properties& 
ScrollingText::properties::set_sleep_time(int sleep_time)
{
    _sleep_time = sleep_time;
    return *this;
}

/*
inline ScrollingText::properties& 
ScrollingText::properties::set_bg_color(Gdk::Color& color)
{
    _bgColor = color;
    return *this;
}

inline ScrollingText::properties& 
ScrollingText::properties::set_fg_color(Gdk::Color& color)
{
    _fgColor = color;
    return *this;
}

inline const Gdk::Color& ScrollingText::properties::get_bg_color() const
{
    return _bgColor;
}

inline const Gdk::Color& ScrollingText::properties::get_fg_color() const
{
    return _fgColor;
}
*/
inline int ScrollingText::properties::get_sleep_time() const
{
    return _sleep_time;
}

inline int  ScrollingText::properties::get_scrolling_step  () const
{
    return _scrolling_step;
}

inline ScrollingText::properties&
ScrollingText::properties::set_scrolling_step  (int scrolling_step)
{
    _scrolling_step = scrolling_step;
    return *this;
}

inline void ScrollingText::reset()
{
    set_text("");
}

inline void ScrollingText::set_scrolling_step  (int step)
{
    _props.set_scrolling_step(step);
}

inline int ScrollingText::get_cur_pos() const
{
    return _pos;
}

inline void ScrollingText::decrease_pos()
{
    set_cur_pos(get_cur_pos()-get_scrolling_step());
}

inline int ScrollingText::get_text_width() const
{
    return _text_width;
}

inline bool ScrollingText::is_text_visible() const
{
    return (get_cur_pos() > -get_text_width());
}

inline void ScrollingText::init_cur_pos()
{
    set_cur_pos(get_allocation().get_width());
}

inline void ScrollingText::set_cur_pos(int value)
{
    _pos = value;
}

inline Glib::ustring ScrollingText::get_text()
{
    Glib::RecMutex::Lock lock(mutex);
    return get_layout()->get_text();
}

inline const Gdk::Color& ScrollingText::get_bg_color() const
{
    
    return (no_const(this)->_bgColor = 
            get_parent()->get_style()->get_bg(get_state()));
}

inline int ScrollingText::get_scrolling_step () const
{
    return _props.get_scrolling_step();
}

inline const Gdk::Color& ScrollingText::get_fg_color() const
{
    return (no_const(this)->_fgColor =  
        get_parent()->get_style()->get_fg(get_state()));
}

inline int ScrollingText::get_sleep_time() const
{
    return _props.get_sleep_time();
}
 
inline void ScrollingText::scroll(int value)
{
    set_cur_pos (get_cur_pos() + value);
    repaint();
}
   
inline void ScrollingText::set_pause(bool pause)
{
    _paused = pause;
}

inline Glib::RefPtr<Gdk::Pixmap> ScrollingText::get_pixmap()
{
    return _pixmap;
}

inline void ScrollingText::set_stop_on_mouse_over(bool flag)
{
    _stop_on_mouse_over = flag;
}

}
#endif // _GtkExtensions_ScrollingText_
