/*
--------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
--------------------------------------------------------------------------------
*/
#include "scrollingtextex.hh"
namespace GtkExtensions 
{

ScrollingTextEx::ScrollingTextEx
            (const ScrollingTextEx::properties& props, callback* pcallback):
                ScrollingText(props),
                _callback_ptr (pcallback)
{
    init();
}

ScrollingTextEx::ScrollingTextEx(callback* pcallback):
                ScrollingText(ScrollingTextEx::properties()),
                _callback_ptr (pcallback)
{
    init();
}

ScrollingTextEx::ScrollingTextEx
            (const ScrollingTextEx::properties& props):
                ScrollingText(props),
                _callback_ptr (this)
{
    init();
}

void ScrollingTextEx::init()
{
	signal_button_press_event().connect(
		    sigc::mem_fun(*this, &ScrollingTextEx::on_button_press_event));
	signal_motion_notify_event().connect(
		    sigc::mem_fun(*this, &ScrollingTextEx::on_mouse_over));
	add_events(Gdk::BUTTON_PRESS_MASK |Gdk::POINTER_MOTION_MASK);
}

bool ScrollingTextEx::on_button_press_event (GdkEventButton* event)
{
	if(event->button == LEFT_BUTTON && 	event->type == GDK_2BUTTON_PRESS)
	{
		parts::iterator it= find_part(int (event->x));
		if (it != _parts.end())
		{
            Glib::RefPtr<Gdk::Window> window = get_window();
            window->set_cursor(Gdk::Cursor(Gdk::WATCH));
			//printf("pressed %s\n", it->second->get_text().c_str());
			it->second->action();
            window->set_cursor();
        	return true;
		}
	}
	return false;
}

bool  ScrollingTextEx::on_mouse_over (GdkEventMotion* event)
{
    parts::iterator it = find_part(int(event->x));
    if (it != _parts.end())
	{
        if (_last_part != it->second)
        {
            it->second->over();
            _last_part = it->second;
        }
	}
	return false;
}

ScrollingTextEx& 
ScrollingTextEx::add_part(boost::shared_ptr<ScrollingTextEx::part> part_arg)
{
    printf ("ScrollingTextEx::add_part %s\n", part_arg->get_text().c_str());
    set_text(get_text()+ get_separator() + part_arg->get_text());
    _parts.push_back(pair (get_text_width(), part_arg));
    return (*this);
}

void ScrollingTextEx::set_cur_pos(int value)
{
    ScrollingText::set_cur_pos(value);
}

ScrollingTextEx::parts::iterator ScrollingTextEx::find_part(int pos)
{
    for (parts::iterator it =_parts.begin(); it != _parts.end(); it++)
    {
        if (-get_cur_pos() + pos < it->first )
        {
            return it;
        }
    }
    return _parts.end();
}

void ScrollingTextEx::init_cur_pos()
{
    printf("ScrollingTextEx::init_cur_pos()\n");
    ScrollingText::init_cur_pos();
    _callback_ptr->restart();      
}

void ScrollingTextEx::reset()
{
    ScrollingText::reset();
    _parts.clear();
}


}
