/*
--------------------------------------------------------------------------------
    Copyright  (C) 2004, Oreste Notelli <oreste.notelli@sischema.com>          
                                                                            
    This program is free software; you can redistribute it and/or modify      
    it under the terms of the GNU General Public License as published by      
    the Free Software Foundation; either version 2 of the License, or         
    (at your option) any later version.                                       
                                                                            
    This program is distributed in the hope that it will be useful,           
    but WITHOUT ANY WARRANTY; without even the implied warranty of            
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
    GNU Library General Public License for more details.                      
                                                                            
    You should have received a copy of the GNU General Public License     
    along with this program; if not, write to the 
    Free Software Foundation, Inc.,                                       
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             

    Sischema s.r.l.
    via Sebenico, 22
    20124 Milano - Italia
    tel-fax: (+39)0266823882
--------------------------------------------------------------------------------
*/
#ifndef _GtkExtensions_ScrollingTextEx_
#define _GtkExtensions_ScrollingTextEx_

#include "scrollingtext.hh"
#include <boost/shared_ptr.hpp>
#include <vector>
#include <utility>

namespace GtkExtensions 
{
class callback
{
public:
    virtual void restart() = 0;
    virtual ~callback(){};
};

class emptycallback: public callback
{
    virtual void restart() {};
};

class ScrollingTextEx : public ScrollingText, public emptycallback
{
public:
    /*
     * ScrollingTextEx::callback interface.       
     */
    typedef callback callback;

    /*
     * ScrollingTextEx::part interface.
       
     */
    class part
    {
    public:
        virtual Glib::ustring   get_text() const = 0;
        virtual void            over() = 0;
        virtual void            action() = 0;
        virtual ~part(){};
    };

    typedef boost::shared_ptr<part> part_ptr;
    typedef std::pair<int, part_ptr > pair;
    typedef std::vector< pair > parts;
    
    ScrollingTextEx(const ScrollingTextEx::properties& props = 
                    ScrollingTextEx::properties());
    ScrollingTextEx(const ScrollingTextEx::properties& props,
                    callback* pcallback);
    ScrollingTextEx(callback* pcallback);
    ScrollingTextEx&    add_part			(boost::shared_ptr<part> part_arg);
    ScrollingTextEx&    add_part			(part* part_arg);
    // overrides
    virtual void    reset();
protected:
    // overrides
    virtual void	set_cur_pos				(int value);
    virtual void    init_cur_pos();
private:
	bool 			on_button_press_event	(GdkEventButton* event);
	bool 			on_mouse_over			(GdkEventMotion* event);
    parts::iterator	find_part    			(int pos);
    Glib::ustring   get_separator			() const;
    void            init                    ();
	// data
    parts                           _parts;
    callback *                      _callback_ptr;
    part_ptr                        _last_part;
};

//INLINES

inline ScrollingTextEx& ScrollingTextEx::add_part(part* part_arg)
{
    return add_part(boost::shared_ptr<ScrollingTextEx::part> (part_arg));
}

inline Glib::ustring
ScrollingTextEx::get_separator() const
{
    return Glib::ustring("   ");
}

const int LEFT_BUTTON = 1;

}
#endif // _GtkExtensions_ScrollingTextEx_
